import java.util.*;

public class Esame{
  
  
  private static class Sensore implements Comparable<Sensore>{
    public double costo; //costo
    public String nome; //nome
	public String tipo; //tipo

	/**
	* Costruttore con 3 argomenti
	*
	* @param nome nome del sensore
	* @param tipo tipo del sensore
	* @param costo costo del sensore
	*/
    public Sensore(String n, String t, double c){
      nome = n;
      tipo = t;
	  costo = c;	
    }

    public String toString(){
	    return "<"+nome+" "+tipo+" "+costo+">";
    }


    public int compareTo(Sensore s){
			int diff = Integer.signum((int)(costo-s.costo)); 
			if (diff!=0) {
				return diff;
			} else {
				return (nome.compareTo(s.nome)==0)?Integer.signum(tipo.compareTo(s.tipo)):Integer.signum(nome.compareTo(s.nome)); 
						
			}
    }


    public boolean equals(Object o){
		if (o instanceof Sensore){
			Sensore s = (Sensore) o;
			return nome.equals(s.nome) && (costo==s.costo) && (tipo.equals(s.tipo));		
		} 
		return false;
    }

    /**
    * Implementa il metodo hashCode per oggetti di tipo Sensore. (Punti 20)
    * il metodo hashCode deve ritornare un valore consistente rispetto all'implementazione del metodo equals: se il metodo equals per due sensori s1 ed s2 ritorna true il metodo hashCode deve ritornare lo stesso valore se invocato su s1 ed s2.
    * @return un intero che rappresenta l'hash code dell'oggetto di invocazione
    */
	public int hashCode() {
		Double oc = new Double(costo);
		return (nome.hashCode()+oc.hashCode()+tipo.hashCode());
	}
	
 } 

  
  /**
  * data una matrice di double, ritorna il numero di righe che contengono solo elementi strettamente positivi (Punti 25) 
  * Ad esempio data la matrice 
  * {{ 0.3, 0.7, 0.8 },
  *  { 0.5, 0.9, -0.5 },
  *  { 0.2, 0.1, 0.6 }}
  * il metodo ritorna il valore 2 (prima e ultima riga)
  *
  * 
  *  @param m la matrice di double in ingresso
  *  @return un intero che rappresenta il numero di righe che contengono solo elementi strettamente positivi
  */	
  public static int numPosRows(double[][] m){
	int posRow = 0;
	for (int r=0; r<m.length;r++){		
	    boolean countRow = true;		
		for (int c=0; c<m[r].length && countRow; c++){
			if (m[r][c] < 0){
				countRow = false;
			}
		} 
		if (countRow) {
			posRow++;
		}
	}

    return posRow;
  }
  
 
  /**
  * Data una string contenente caratteri qualsiasi, restituire una mappa che associa un carattere al numero di occorrenze per quel carattere nella stringa data. (Punti 25)
  * Ad esempio se s = "AAAGG" la mappa da restituire sara' {A=3, G=2} .
  *
  * @param s stringa di caratteri qualsiasi
  * @return mappa che associa i caratteri presenti nella stringa al loro numero di occorrenze.
  */
  public static Map<Character,Integer> stringToMap(String s){
    Map<Character,Integer> res = new HashMap<Character,Integer>();
	char[] c = s.toCharArray();
	for (int i=0; i<c.length; i++){
		if (res.keySet().contains(c[i])){
			res.put(c[i],res.get(c[i])+1);		
		} else {
			res.put(c[i],1);
		}		    
    }
    return res;
  }

  /**
  * Dato un grafo ed un nodo n ritornare l'insieme di tutti i nodi non raggiungibili dal nodo n (Punti 30)
  *
  * @param g grafo in ingresso
  * @param n nodo in ingresso
  * @return insieme dei nodi non raggiungibli da n
  */
  public static Set<Object> notReachSet(Grafo g, Object n){
	Set<Object> visited = new HashSet<Object>();
	dfs(g,n,visited);
	Set<Object> res = new HashSet<Object>(g.getNodeSet());	
    res.removeAll(visited);
	return res;
  }

  public static void dfs(Grafo g, Object node, Set<Object> visited){
	visited.add(node);
	for(Arco a : g.getEdgeSet(node)){
		if (!visited.contains(a.getNode1())){
			dfs(g,a.getNode1(),visited);		
		}
		if (!visited.contains(a.getNode2())){
			dfs(g,a.getNode2(),visited);		
		}
	}
  }

  public static void main(String[] args){

 
    System.out.println("**********TEST hashCode per Sensore*************");	

	Sensore s1 = new Sensore("a","ph",35.2);
	Sensore s2 = new Sensore("a","ph",35.2);
	Sensore s3 = new Sensore("b","ph",35.2);
	Set<Sensore> test = new HashSet<Sensore>();

	test.add(s1); 
	test.add(s2); 
	test.add(s3); 

	Sensore s3copy = new Sensore("b","ph",35.2);

	System.out.println("valore hash del sensore "+s1+" comparato con "+s2+" : "+ (s1.hashCode() - s2.hashCode())+" [dovrebbe essere zero])");

	System.out.println("L'insieme " +test+ " contiene il sensore "+s3copy+" ?: " + test.contains(s3copy) + " [dovrebbe essere true])");

    
    System.out.println("**********TEST numPosRows()*************");

	double[][] m1= {
      { 0.3, 0.7, 0.8 },
      { 0.5, 0.9, -0.5 },
      { 0.2, 0.1, 0.6 },
    };

	double[][] m2= {
      { -0.5, -0.9, 0.5 },
      { 0.3, 0.3, -0.9 },
    };


    System.out.println("matrice  in ingresso \n "+Arrays.deepToString(m1)+" \n numero di righe : "+ numPosRows(m1)+" [dovrebbe essere 2]");

    System.out.println("matrice  in ingresso \n "+Arrays.deepToString(m2)+" \n numero di righe : "+ numPosRows(m2)+" [dovrebbe essere 0]");
      
    System.out.println("**********TEST stringToMap()*************");
 
	String str1 = "AAAGG";
	String str2 = "AAAAAATTTG";

	Map<Character,Integer> map1 = stringToMap(str1);
	Map<Character,Integer> map2 = stringToMap(str2);

    System.out.println("mappa per la stringa "+str1+" : "+map1+" dovrebbe essere {A=3, G=2}");

    System.out.println("mappa per la stringa "+str2+" : "+map2+" dovrebbe essere {A=6, T=3, G=1}");

	
    System.out.println("**********TEST reachSet()*************");
 
	    Object na = "a"; Object nb = "b"; Object nc = "c"; Object nd = "d"; Object ne = "e"; Object nf = "f"; Object nh = "h"; Object nk = "k";

	    Grafo g = new Grafo("g");    
	    g.add(na,nb,0);
	    g.add(nb,nc,0);
	    g.add(nc,nd,0);
	    g.add(nc,ne,0);
	    g.add(nh,nk,0);
	    
	    System.out.println("L'insieme di archi per il grafo g: " + g.getEdgeSet());
		

	   System.out.println("Nodi non raggiungibili a partire dal nodo a " + notReachSet(g,na) +" [dovrebbe essere [h, k]]");

	   System.out.println("Nodi non raggiungibili a partire dal nodo h " + notReachSet(g,nh) +" [dovrebbe essere [a, b, c, d, e]]");
   
    
  }

}
